/********************************************************************************/
/*                                                                              */
/*              depend_local.H                                                  */
/*                                                                              */
/*      Local class definitions for dependency analyzer                         */
/*                                                                              */
/********************************************************************************/


#ifndef DEPEND_LOCAL_ALREADY_DEFINED
#define DEPEND_LOCAL_ALREADY_DEFINED

#include <stdlib.h>
#include <ctype.h>
#include <iostream.h>

#include <bstring.h>
#include <list.h>
#include <map.h>
#include <vector.h>
#include <function.h>



/********************************************************************************/
/*                                                                              */
/*      Parameter Definitions                                                   */
/*                                                                              */
/********************************************************************************/


const   int             MAX_LINE_LENGTH = 2048;



/********************************************************************************/
/*                                                                              */
/*      Local Type Definitions                                                  */
/*                                                                              */
/********************************************************************************/


typedef const char *    ConstText;
typedef char            Character;
typedef int             Integer;
typedef string          String;
typedef bool            Boolean;



/********************************************************************************/
/*                                                                              */
/*      Forward Definitions                                                     */
/*                                                                              */
/********************************************************************************/


typedef class DependNodeInfo *                          DependNode;
typedef class DependNodeSetInfo *                       DependNodeSet;

typedef list<DependNode>                                DependNodeList;
typedef DependNodeList::iterator                        DependNodeListIter;

typedef map<String,DependNode,less<String> >            DependNodeTable;
typedef DependNodeTable::iterator                       DependNodeTableIter;



/********************************************************************************/
/*                                                                              */
/*      Class DependNodeVector -- dynamic vector of DependNodes                 */
/*                                                                              */
/********************************************************************************/


typedef vector<DependNode>                              DependNodeVectorUnsafe;


class DependNodeVector : public DependNodeVectorUnsafe {
public:
   DependNodeVector()                                   { }
   ~DependNodeVector()                                  { }

   DependNode operator[](int v)
      { if (v < 0 || v >= size()) return NULL;
        else return DependNodeVectorUnsafe::operator[](v);
      }
   const DependNode operator[](int v) const
      { if (v < 0 || v >= size()) return NULL;
        else return DependNodeVectorUnsafe::operator[](v);
      }
};


typedef DependNodeVector::iterator                      DependNodeVectorIter;




/********************************************************************************/
/*                                                                              */
/*      Class DependNodeInfo -- node for a dependency                           */
/*                                                                              */
/********************************************************************************/


class DependNodeInfo {

private:
   String node_name;
   Integer num_link;
   DependNodeVector depend_on;

public:
   DependNodeInfo(String);
   ~DependNodeInfo();

   ConstText name() const;
   Integer numLink() const;

   void addLink(DependNode);
   Integer updateLinkCount(Integer by);

   DependNodeVectorIter links();
   DependNodeVectorIter linksEnd();
};



/********************************************************************************/
/*                                                                              */
/*      Class DependNodeSetInfo -- set of all nodes                             */
/*                                                                              */
/********************************************************************************/


class DependNodeSetInfo {

private:
   DependNodeTable node_table;

public:
   DependNodeSetInfo();
   ~DependNodeSetInfo();

   void readNodes(istream&);

   void topSort(ostream&);

private:
   DependNode findNode(String);
};


#endif

/* end of depend_local.H */

