/********************************************************************************/
/*                                                                              */
/*              dependnodeset.C                                                 */
/*                                                                              */
/*      Implementation of node sets                                             */
/*                                                                              */
/********************************************************************************/


#include "depend_local.H"

#include <strstream.h>


/********************************************************************************/
/*                                                                              */
/*      DependNodeSetInfo constructors/destructors                              */
/*                                                                              */
/********************************************************************************/


DependNodeSetInfo::DependNodeSetInfo()                          { }


DependNodeSetInfo::~DependNodeSetInfo()
{
   DependNodeTableIter titer;
   DependNode dn;

   for (titer = node_table.begin(); titer != node_table.end(); ++titer) {
      dn = (*titer).second;
      delete dn;
    }
}



/********************************************************************************/
/*                                                                              */
/*      DependNodeSetInfo file input methods                                    */
/*                                                                              */
/********************************************************************************/


void
DependNodeSetInfo::readNodes(istream& inf)
{
   Character lbuf[MAX_LINE_LENGTH];
   String from,to;
   DependNode f,t;

   while (inf) {
      inf.getline(lbuf,MAX_LINE_LENGTH);
      if (inf.eof()) break;
      if (lbuf[0] == 0 || lbuf[0] == '#') continue;

      istrstream ist(lbuf,strlen(lbuf));
      ist >> from >> to;
      // if (ist.fail()) continue;              // string::<< sets this at eos
      findNode(from)->addLink(findNode(to));
    }
}



/********************************************************************************/
/*                                                                              */
/*      DependNodeSetInfo hash table maintenance methods                        */
/*                                                                              */
/********************************************************************************/


DependNode
DependNodeSetInfo::findNode(String nm)
{
   DependNode dn = node_table[nm];

   if (dn == NULL) {
      dn = new DependNodeInfo(nm);
      node_table[nm] = dn;
    }

   return dn;
}



/********************************************************************************/
/*                                                                              */
/*      DependNodeSetInfo topological sort methods                              */
/*                                                                              */
/********************************************************************************/


void
DependNodeSetInfo::topSort(ostream& ost)
{
   DependNodeList workqueue;
   DependNodeTableIter tbliter;
   DependNode dn,ldn;

   ost << "\nThe dependency order is:" << endl;

   for (tbliter = node_table.begin(); tbliter != node_table.end(); ++tbliter) {
      dn = (*tbliter).second;
      if (dn->numLink() == 0) workqueue.push_back(dn);
    }

   while (!workqueue.empty()) {
      dn = workqueue.front();
      workqueue.pop_front();
      ost << "\t" << dn->name() << endl;

      DependNodeVectorIter liter;
      DependNode ldn;

      for (liter = dn->links(); liter != dn->linksEnd(); liter++) {
         ldn = *liter;
         if (ldn->updateLinkCount(-1) == 0) workqueue.push_back(ldn);
       }
    }
}



/* end of dependnodeset.C */
